library websocket;

import 'dart:async';
import 'dart:html' as html;
import './websocket_stub.dart' as stub;

class WebSocket implements stub.WebSocket {
  html.WebSocket _socket;

  WebSocket._(this._socket) {
    _socket.onClose.listen((html.CloseEvent event) {
      closeCode = event.code;
      closeReason = event.reason;
      _streamController.close();
    });

    _socket.onError.listen((html.Event error) {
      _streamController.addError(error);
    });
    _socket.onMessage.listen((html.MessageEvent message) {
      _streamController.add(message.data);
    });
  }

  static Future<WebSocket> connect(
    String url, {
    Iterable<String> protocols,
  }) async {
    final s = html.WebSocket(url, protocols);
    await s.onOpen.first;
    return WebSocket._(s);
  }

  @override
  void add(/*String|List<int>*/ data) => _socket.send(data);

  @override
  Future addStream(Stream stream) {
    final completer = Completer();
    stream.listen((data) {
      _socket.send(data);
    }, onError: (error) {
      _socket.send(error.toString());
      completer.completeError(error);
    }, onDone: () => completer.complete());
    return completer.future;
  }

  @override
  void addUtf8Text(List<int> bytes) => _socket.send(bytes);

  @override
  Future close([int code, String reason]) {
    if (code != null) {
      _socket.close(code, reason);
    } else {
      _socket.close();
    }
    return _socket.onClose.first;
  }

  @override
  int closeCode;

  @override
  String closeReason;

  @override
  String get extensions => _socket.extensions;

  @override
  String get protocol => _socket.protocol;

  @override
  int get readyState => _socket.readyState;

  @override
  void addError(Object error, [StackTrace stackTrace]) =>
      _streamController.addError(error, stackTrace);

  @override
  Future get done => _socket.onClose.first;

  StreamController<String> _streamController =
      StreamController.broadcast(); //Add .broadcast here

  @override
  Stream<dynamic /*String|List<int>*/ > get stream => _streamController.stream;
}
